<?php

require_once('vb_api_client.php');
require_once('config.php');
require_once('utilities.php');
require_once('editor.php');
require_once('vb_link_helper.php');
//
//    Category
//    
//        A Category is essentially a view of the vBulletin board. The Category may 
//        not be based on a certain content type, the idea is only that it�s 
//        a digest of some data associated with the board.
//
//        These Categories are content streams based off one or more API 
//        (not necessarily VB only) calls.
//
//        Each provides both a cached data array, and specific display adapters to 
//        translate the data type into HTML.
//
//        Categories are configurable. See config.php.new for examples.
//
//        This base class implements most of the functionality required for a Category,
//        most subclasses will implement self::refreshData() and be complete.
//
abstract class Category {

    //
    //    Makes API call(s) to refresh the Category's backing data
    //
    //    Responsible for returning an array of CategorySummaryItems, or an empty
    //    array on error or null data set.
    //
    abstract protected function refreshData();
    
    // Array of CategorySummaryItem
    protected $data;
    // Seconds since epoch when cached (timestamp)
    protected $last_data_time = 0;
    // Seconds to cache data for
    protected $cache_data_for;
    // Editorial weight of category
    protected $weight = 0.1;
    
    //
    //  Factory implementation, registry contains all singleton instances of sub classes.
    //  This is the only entry into the Category heirarchy!
    //
    //  If the category is variadic, it gets instantiated by the factory based on
    //  a category name like MOD_TYPE_FRAGMENT. The idea here is, we may have a category which
    //  shares the exact same functionality, but varies based on some identifier.
    //  An example of this would be the category Forum_Threads_123, where the 123 fragment is a forumid.
    //  Instead of implementing each as a seperate category, only have one class, but
    //  have the factory construct different instances based on the fragment.
    //
    //  Variadic categories by default are not included in the front page or filters.
    //  Explicitly set this up in the implementing category if desired.
    //
    private static $registry;    
    public static function registry($category_name) {      
        if(!isset(self::$registry[$category_name])) {
            $chunk_array = explode('_', $category_name);
            $class_name = get_class() . "_" . $category_name;      

            //
            //  If class is variadic (class name of the form Category_X_Y_Z),
            //  get the Z part to send to the Category_X_Y implementation
            //
            
            $fragment = '';
            if(count($chunk_array) > 2) {
                $fragment = $chunk_array[2];
            }
            
            //
            //  Get the correct implementation class name, in the typical case
            //  this is Category_X_Y, we leave room for strange cases though.
            //
            
            if(count($chunk_array) > 1) {
                $class_name = get_class() . "_" . $chunk_array[0] . "_" . $chunk_array[1];
                $generaltype = $chunk_array[0] . "_" . $chunk_array[1];
            } else {
                $class_name = get_class() . "_" . $category_name;
                $generaltype = $category_name;
            }

            if(class_exists($class_name)) {
            
                //
                //    Check if we have config values to send to this Category,
                //    if not, instantiate with default values.
                //    
                
                if(!empty($GLOBALS['config']['categories'][$category_name])) {
                    self::$registry[$category_name] = new $class_name(  $fragment,
                                                                        $GLOBALS['config']['categories'][$generaltype]['weight'], 
                                                                        $GLOBALS['config']['categories'][$generaltype]['cache_time'],
                                                                        $GLOBALS['config']['categories'][$generaltype]['relevant_period'],
                                                                        $GLOBALS['config']['categories'][$generaltype]['color']);
                } else {
                    self::$registry[$category_name] = new $class_name($fragment);
                }
            }
        }
        return self::$registry[$category_name];
    }    
        
    private static $all_init = true;
    private static $all_blacklist = array(
        "Search_All",
        "Search_Threads",
        "Search_Blogs",
        "Search_Articles",
        "Forum_Threads"
    );
    private static $v3_blacklist = array(
        "Recommended_Threads",
        "Recommended_All",
        "Friends_All",
        "Friends_Threads",
        "Friends_Blogs",
        "Friends_Articles",
        "Newest_Articles",
        "Popular_Articles",
        "All_Articles"
    );
    
    //
    //  An array of all the eligible front page categories
    //  
    
    public static function first_page() {
        $ret = array();
        $ret2 = array();
        
        $ret[] = "Newest_Threads";
        $ret[] = "Popular_Threads";
        $ret[] = "Subscribed_Threads";
        $ret[] = "Recommended_Threads";
        $ret[] = "Friends_All";
        $ret[] = "Newest_Blogs";
        $ret[] = "Popular_Blogs";
        $ret[] = "Newest_Photos";
        $ret[] = "Popular_Photos";
        $ret[] = "Newest_Articles";
        $ret[] = "Popular_Articles";
        
        foreach($ret as $category_name) {
            if(VBApiClient::instance()->getVBMajorVersion() == 3 && in_array($category_name, self::$v3_blacklist)) {
                continue;
            }
            $ret2[] = self::registry($category_name);
        }
        return $ret2;
    }  
    
    //
    //  Get an array of all non-abstract class implementations of Category.
    //    Certain classes are excluded through a blacklist.
    //
    
    public static function all() {
        if(self::$all_init) {
            self::$all_init = false;
            $php_code = file_get_contents('category.php');
            $classes = array();
            $tokens = token_get_all($php_code);
            $count = count($tokens);
            for ($i = 4; $i < $count; $i++) {
                if( $tokens[$i - 4][0] != T_ABSTRACT &&
                    $tokens[$i - 3][0] == T_WHITESPACE &&
                    $tokens[$i - 2][0] == T_CLASS && 
                    $tokens[$i - 1][0] == T_WHITESPACE && 
                    $tokens[$i][0] == T_STRING) {
                    $class_name = $tokens[$i][1];
                    $classes[] = $class_name;
                }
            }
            
            foreach($classes as $class_name) {
                if(strstr($class_name, "Category_")) {
                    $category_name = str_replace("Category_", "", $class_name);
                    if(!in_array($category_name, self::$all_blacklist)) {
                        if(VBApiClient::instance()->getVBMajorVersion() == 3 && in_array($category_name, self::$v3_blacklist)) {
                            continue;
                        }
                        self::registry($category_name);
                    }
                }
            }
        }
        return self::$registry;
    }
    
    public static function filter($selected_mod = 'All', $selected_type = 'All') {           
        $used_categories = array();
        $categories = Category::all();
        $criteria = ($selected_mod == 'All' ? '' : $selected_mod) . '_' . ($selected_type == 'All' ? '' : $selected_type);
        
        foreach($categories as $category) {
            if(strstr($category->getCategoryName(), $criteria)) {
                $used_categories[] = $category;               
            }
        }
        return $used_categories;
    }
    
    // Can sync operations on this timestamp for this request! Use self::now() in subclasses!
    
    private static $now;
    protected static function now() {
        if(!isset(self::$now)) {
            self::$now = time();
        }
        return self::$now;
    }
    
    protected function __construct($fragment = '', $weight = 0.5, $cache_data_for = 25, $relevant_period = 2592000, $color = '000000') {
        $this->cache_data_for = $cache_data_for;      
        $this->weight = $weight;
        $this->relevant_period = $relevant_period;
        $this->color = $color;
        $this->fragment = $fragment;
        $this->getCachedData();
    }
    
    //
    //  Cannonical category identifier, usually of the form MOD_TYPE,
    //  e.g. Friends_Threads. Some categories are variadic, and take
    //  the form MOD_TYPE_FRAGMENT. The possibility exists for an
    //  implementation outside this frame, but is not currently used.
    //
    public function getCategoryName() {
        return str_replace("Category_", "", get_class($this)) . ((!empty($this->fragment)) ? ('_' . $this->fragment) : '');
    }
    
    //
    //     By default, all non-variadic implementation classes are shown in the second page
    //
    public function showAsFilter() {
        return empty($this->fragment);
    }
    
    //
    //    $data is cached in this scope, other persisted data may also
    //    use this scope, if careful to avoid 'data' and 'last_data_time' keys.
    //
    protected function sessionNamespace() {
        return 'vbfb_category_cache::' . $this->getCategoryName();
    }
    
    //
    //  By default this is based off the category name, and uses the second part.
    //  (The TYPE in MOD_TYPE)
    //
    public function getContenttype() {
        $category_name_chunks = @explode("_", $this->getCategoryName());
        $typical =  @$category_name_chunks[1];
        if(isset($typical)) {
            return $typical;
        } else {
            return 'Unknown';
        }
    }
        
    public function getColor() {
        return "#" . $this->color;
    }
    
    public function getDisplayName() {
        return str_replace("_", " ", $this->getCategoryName());
    }
    
    // + View code +
    
    //
    //     Gets the container html for this Category
    //
    public function getDashboardContainerHtml($inner) {        
        $inner = (!empty($inner) ? $inner  : $this->getNoDataHtml());
        $classHide = "";
        $aux = mb_substr($inner,-9);
	    
        if($aux == "__-__hide"){
            $inner = mb_substr($inner,0,-9);
            $classHide = "hide_category";
        }
	
        return '<div class="category_box ' . $classHide . '" data-category="'.$this->getCategoryName().'">
                    ' . $this->getDashboardContainerHeaderHtml() . '
                    <ul class="'.get_class($this).'">
                    ' . $inner . '
                    </ul>
                    ' . $this->getDashboardContainerHeaderDropShadowHtml() . '
                </div>';
    }
    
    //
    //     Gets the container html for this Category, when placed next to an ad
    //
    public function getSmallDashboardContainerHtml($inner) {
        $inner = (!empty($inner) ? $inner  : $this->getNoDataHtml());
        return '<div style="height: 244px;" class="small category_box" data-category="'.$this->getCategoryName().'">
                    ' . $this->getDashboardContainerHeaderHtml() . '
                    <ul class="'.get_class($this).'">
                    ' . $inner . '
                    </ul>
                    ' . $this->getDashboardContainerHeaderDropShadowHtml() . '
                </div>';
    }
    
    //
    //     Gets the container html for this Category, when used for 
    //    displaying a single content item
    //
    public function getSingleItemContainerHtml($inner) {
        return $this->getDashboardContainerHtml($inner);
    }
    
    //
    //     Gets the container html for this Category, when displayed in stream
    //
    public function getStreamContainerHtml($inner) {
        return $inner;
    }
    
    protected function getDashboardContainerHeaderHtml() {        
        if(empty($this->fragment)) {
            list($first, $second) = explode(" ", $this->getDisplayName());
            $titlehtml = '<span class="first">'.$first.'</span>'. (isset($second) ? '<span class="second">'.$second.'</span>' : '');
        } else {
            $titlehtml = '<span>'.$this->getDisplayName().'</span>';
        }
        
        return '<div class="category_header">
                    <a href="category_stream.php?category='.$this->getCategoryName().'">
                        <div class="icon" style="background: white url(\''.$this->getIcon().'\') no-repeat top left;"></div>
                        <div class="text">'. $titlehtml .'</div>
                        <div class="more">more</div>
                    </a>
                </div>';
    }
    protected function getDashboardContainerHeaderDropShadowHtml() {
        return '<div class="category_shadow">
                    <div class="category_shadow_left"></div>
                    <div class="category_shadow_center"></div>
                    <div class="category_shadow_right"></div>
                </div>';
    }
        
    protected function getNoDataHtml() {
        return '<li class="nodata">Nothing to display.</li>';
    }
    
    protected function getIcon() {
        list($mod, ) = explode("_", $this->getCategoryName());
        return '_img/ico_'.strtolower($mod).'.png';
    }
    
    //
    //     Returns the HTML which will be rendered in the Dashboard
    //
    public final function getDashboardHtml($list) {
	if(count($list) == 1 && array_key_exists('display_category', $list)) return;
        $html = '';
        $hideCategory = false;
        if(array_key_exists('display_category', $list)){ $list = array_slice($list,-2,1); $hideCategory = true;}
        foreach($list as $dataitem) {
            $html = $html . $dataitem->getDashboardHtml();
        }
	
        if($hideCategory) $html = $html . '__-__hide';
        return $this->getDashboardContainerHtml($html);
    }
    
    //
    //     Returns the HTML which will be rendered in the Dashboard, 
    //    when placed next to an ad
    //
    public final function getSmallDashboardHtml($list) {
        $html = '';
        foreach($list as $dataitem) {
            $html = $html . $dataitem->getSmallDashboardHtml();
        }
        return $this->getSmallDashboardContainerHtml($html);
    }
    
    // - View code -
    
    // + Data code +
    
    public final function getData() {
        $this->doRefresh();
        return $this->data;
    } 
    
    
    public final function expired() {
        return ((self::now() - $this->last_data_time) > $this->cache_data_for);
    }
    
    public final function expiresAt() {
        return $this->last_data_time + $this->cache_data_for;
    }
    
    
    //
    //    Default caching implementation, override these three methods to
    //    implement custom behavior.
    //        see Category_Paged for an example
    //
    
    //
    //    This method should be called whenever user input updates a Category
    //        e.g. a user posts a new thread, or sends a reply, the 
    //        *_Threads Categories should be invalidated.
    //
    public function invalidate() {
        $this->last_data_time = 0;
        $this->data = null;
        $_SESSION[$this->sessionNamespace()]['last_data_time'] = $this->last_data_time;
        $_SESSION[$this->sessionNamespace()]['data'] = $this->data;
    }
    
    protected function getCachedData() {
        if(isset($_SESSION[$this->sessionNamespace()])) {
            $cat_session = $_SESSION[$this->sessionNamespace()];
            $this->data = $cat_session['data'];
            $this->last_data_time = $cat_session['last_data_time'];
        }
    }
    protected function setData($data) {
        $this->last_data_time = self::now();
        $this->data = $data;
        $_SESSION[$this->sessionNamespace()]['last_data_time'] = $this->last_data_time;
        $_SESSION[$this->sessionNamespace()]['data'] = $this->data;
    }            
 
    protected final function getCacheDataFor() {
        return $this->cache_data_for;
    }

    private function doRefresh() {
        if($this->expired() || empty($this->data)) {
            if(!empty($this->data) && VBApiClient::instance()->timeInAPIThisRequest() > 0.6) {
                return;
            }
            $data = $this->refreshData();            
            if($data == null) {
                $data = array();
            }
            usort($data, array("Category", "data_item_sorter"));

            $this->setData($data);
        } 
    }
    
    private function data_item_sorter($data_item1, $data_item2) {
        $weight1 = $data_item1->getRelevance();
        $weight2 = $data_item2->getRelevance();
        if($weight1 > $weight2) return -1;
        else if($weight1 == $weight2) {
            $weight1 = self::registry($data_item1->getCategory())->getWeight();
            $weight2 = self::registry($data_item2->getCategory())->getWeight();
            
            if($weight1 > $weight2) return -1;
            else if($weight1 == $weight2) return 0;
            else return 1;
        }
        else return 1;
    }
    
    //
    //  Return a timestamp at which the set of passed categories (or all categories)
    //  will need to be refreshed.
    //    
    public static function expires($categories = null) {
        if(!isset($categories)) {
            $categories = self::all();
        }
        $min = 1000000000000000;
        foreach($categories as $category) {
            $expires = $category->expiresAt();
            if($expires < $min) {
                $min = $expires;
            }
        }
        return $min;
    }
    
    // - Data code -
    
    // + Relevancy code +
    
    //
    //    Return the time period in seconds over which we expect this
    //    category to be relevant.
    //    
    public final function getRelevantPeriod() {
        return $this->relevant_period;
    }
    
    //
    //    Return the ad hoc, editorial weight for this Category
    //
    public final function getWeight() {
        return $this->weight;
    }

    //
    //    Get this Category's dashboard relevance, used to choose the 
    //    placement on the front page. This method is passed a sorted list of data items. 
    //    It will return the aggregate relevancy.
    //
    //    The formula for this is:
    //
    //        relevance = category.weight * (normalize(sum(individual_data_item_relevancies)))
    //
    //        The normalization is weighted to put more significance on the first item
    //        over the second, and the second item over the third.
    //
    public final function getRelevance($list) {                   
        $drop_off_constant = 1.75;
        $per_item_sum_relevance = 0;
        $i = 0;
        $normalize_to = 0;
        foreach($list as $dataitem) {
            $per_item_sum_relevance += ($dataitem->getRelevance() * (1 / (($i * $drop_off_constant) + 1)));
            $normalize_to += (1 / (($i * $drop_off_constant) + 1));
            $i++;
        }
        if($i == 0)
            return 0;
        $item_relevance = $per_item_sum_relevance / $normalize_to;
        return $item_relevance * $this->getWeight();
    }
    
    // - Relevancy code -
}

//
//    CategorySummaryItem
//
//        This class represents a discrete data item returned by a Category.
//        It has two responsibilities, parse the json representation of the
//        returned data set from Category::refreshData(), and implement
//        the view code to represent each data item in that set. Additionally,
//        this class handles the core relevancy algorithm.
//        
abstract class CategorySummaryItem {
    protected $id;
    protected $username;
    protected $userid;
    protected $title;
    protected $viewcount;
    protected $category;
    protected $updated_timestamp;
    protected $json;
    
    public function __construct($json) {
        $this->json = $json;
        $this->category = str_replace("CategorySummaryItem_", "", get_class($this));
        $this->parseJson($this->json);
    }
    
    public final function getId() {
        return $this->id;
    }
    public final function getUsername() {
        return $this->username;
    }
    public final function getUserId() {
        return $this->userid;
    }
    public final function getTitle() {
        return $this->title;
    }
    public final function getViewCount() {
        return $this->viewcount;
    }
    public final function getUpdatedTimestamp() {
        return $this->updated_timestamp;
    }
    public function getCategory() {
        return $this->category;
    }
    public final function getAvatarUrl() {
        $userinfo = UserInfo::get($this->getUsername());
        return $userinfo->getAvatarUrl();
    }
    public final function getProfileUrl() {
        $userinfo = UserInfo::get($this->getUsername());
        return $userinfo->getProfileUrl();
    }
    public final function getJson() {
        return $this->json;
    }
    
    private static $now;
    protected static function now() {
        if(!isset(self::$now)) {
            self::$now = time();
        }
        return self::$now;
    }
    
    public abstract function parseJson($json);
    
    public function getRelevance() {
        $minimum = 0.0000001;
        
        //
        //     Exponential decay based on this category's expected time period
        //
        $ln2 = 0.693147;
        $lambda = ($ln2) / (Category::registry($this->getCategory())->getRelevantPeriod() / 2);
        $seconds_old = self::now() - $this->getUpdatedTimestamp();
        
        $time_relevancy = exp(-(($lambda) * ($seconds_old)));
        
        //
        //    If this summary item has an associated forum,
        //    do subscription based relevancy. Note that this depends on
        //    subclasses using forumid as the field to store the forum id.
        //
        if(isset($this->forumid)) {
            $forum = ForumInfo::get($this->forumid);
            $forum_relevancy = ($forum->isSubscribed() || $forum->hasSubscribedThread()) ? 1 : 0;
            $final_relevancy = ((4 * $time_relevancy) +  ($forum_relevancy)) / 5;
        } else {
            $final_relevancy = $time_relevancy;
        }
        
        return $final_relevancy > $minimum ? $final_relevancy : $minimum;
    }
    
    public function getSmallDashboardHtml() {
        return $this->getSmallStreamHtml();
    }
    
    public function getDashboardHtml() {
        return $this->getStreamHtml();
    }
    
    public function getStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        return '<li class="category_data_item '.$name.' '.get_class().' " data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'">'.$this->getTitle().'</a>
                    </span>
                    <span class="bottom_line">
                    <span class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $this->getUsername() .'</a></span>
                    </span>
                    '. (isset($this->viewcount) ? '
                    <span class="count view">
                        <span class="number">'. $this->viewcount .'</span>
                        <span class="text">views</span>
                    </span>' : '') .'
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function getSmallStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        $title = $this->getTitle();
        return '<li class="category_data_item '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'">'.$title.'</a>
                    </span>
                    <span class="bottom_line">
                    <span class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $this->getUsername() .'</a></span>
                    </span>
                    '. (isset($this->viewcount) ? '
                    <span class="count view">
                        <span class="number">'. $this->viewcount .'</span>
                        <span class="text">views</span>
                    </span>' : '') .'
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function getSingleSidebarHtml($number) {
        $name = Category::registry($this->getCategory())->getCategoryName();        
        return '<li class="category_data_item '.$name.' '.get_class().'" data-category="'.$name.'">
                    <div class="title">
                        <a href="'.$this->getDetailViewLink($name).'">'.$this->getTitle().'</a>
                    </div>
                    <div class="avatarurl"><a href="'.$this->getProfileUrl().'"><img src="'.$this->getAvatarUrl().'" /></a></div>
                    <div class="userinfo">
                        <div class="username"><a href="'.$this->getProfileUrl().'">'.$this->getUsername().'</a></div>
                        <div class="posted">'.timedisplay($this->getUpdatedTimestamp()).'</div>
                    </div>                    
                </li>
                ';
    }
    
    protected function getDetailViewLink($category_name) {
        $page = (!empty($_GET['page']) ? $_GET['page'] : 1);

        if(defined('READ_ONLY') && READ_ONLY) {
            return vB_SITE_ROOT . CategoryContentItem::getRedirect($category_name, $this->getId(), $page);
        } else {
            return 'category_item.php?category='.$category_name.'&id='.$this->getId();
        }
    }
}

//
//    + Category, CategorySummaryItem Implementations +
//

abstract class Category_ApiGetNewTop extends Category {   
    protected final function getMaxItemsPerCategory() {
        return 9;
    }
    protected final function getUpdatedCutoff() {
        return (self::now() - $this->getRelevantPeriod());
    }
}
abstract class CategorySummaryItem_ApiGetNewTop extends CategorySummaryItem {
    protected $avatarurl;
    public function parseJson($json) {
        $this->viewcount = $json['viewcount'];
        $this->username = $json['username'];
        $this->userid = $json['userid'];
        $this->title = $json['title'];
        $this->avatarurl = $json['avatarurl'];
        
        $userinfo = UserInfo::get($this->username);
        if(!empty($this->avatarurl)) { 
            $userinfo->setVBAvatarUrl($this->avatarurl);
    }
        if(!empty($this->userid)) { 
            $userinfo->setVBUserid($this->userid);
        }
        UserInfo::set($userinfo);
    }
    
    public function getStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        return '<li class="category_data_item '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'">'.$this->getTitle().'</a>
                    </span>
                    
                    <div class="bottom_line">
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $this->getUsername() .'</a></div>
                    </div>
                    <span class="count view">
                        <span class="number">'. $this->getViewCount() .'</span>
                        <span class="text">views</span>
                    </span>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function getSmallStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        $title = $this->getTitle();
        return '<li class="category_data_item small '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'">'.$title.'</a>
                    </span>                
                    <div class="bottom_line">
                        <a href="'.$this->getProfileUrl().'">
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by '. $this->getUsername() .'</div>
                        </a>
                    </div>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
}

abstract class CategorySummaryItem_ApiGetNewTop_Threads extends CategorySummaryItem_ApiGetNewTop {
    protected $replycount;
    protected $forumtitle;
    protected $forumid;
    public final function getForumId() {
        return $this->forumid;
    }
    public final function getReplyCount() {
        return $this->replycount;
    }
    public final function getForumTitle() {
        return $this->forumtitle;
    }
    
    public function getStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        $title = $this->getTitle();
        $poster = $this->getUsername();
        return '<li class="category_data_item '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'&goto=newpost">'.$title.'</a>
                    </span>
                    
                    <div class="bottom_line">
                        <div class="teal_box">
                            <div class="teal_box_title">in</div><div class="teal_box_arrow"><div></div></div>
                            <div class="teal_box_text"><a href="category_stream.php?category=Forum_Threads_'.$this->getForumId().'">'.$this->getForumTitle().'</a></div>
                        </div>
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $poster .'</a></div>
                    </div>
                    <span class="count view">
                        <span class="number">'. $this->getViewCount() .'</span>
                        <span class="text">views</span>
                    </span>
                    <span class="count reply">
                        <span class="number">'. $this->getReplyCount() .'</span>
                        <span class="text">replies</span>
                    </span>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function getSmallStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        
        $threadTitle = $this->getTitle();
        $forumTitle = $this->getForumTitle();
        $poster = $this->getUsername();
        return '<li class="category_data_item small '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'&goto=newpost">'.$threadTitle.'</a>
                    </span>                
                    <div class="bottom_line">
                        <div class="teal_box">
                            <div class="teal_box_title">in</div><div class="teal_box_arrow"><div></div></div>
                            <div class="teal_box_text"><a href="category_stream.php?category=Forum_Threads_'.$this->getForumId().'">'.$forumTitle.'</a></div>
                        </div>
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $poster .'</a></div>
                    </div>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function parseJson($json) { 
        parent::parseJson($json);
        $this->id = $json['id']; 
        $this->updated_timestamp = $json['lastposttime'];
        $this->username = $json['lastpostuser'];
        $this->replycount = $json['replycount'];
        $this->forumtitle = $json['forumtitle'];
        $this->forumid = $json['forumid'];
        
        $forum = ForumInfo::get($this->forumid);
        $forum->setName($this->forumtitle);
        ForumInfo::set($forum);
        $thread = ThreadInfo::get($this->id);
        $thread->setName($this->title);
        ThreadInfo::set($thread);
    }
}

final class CategorySummaryItem_Newest_Threads extends CategorySummaryItem_ApiGetNewTop_Threads {
}

final class Category_Newest_Threads extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['new']) && isset($json['new']['thread']) && count($json['new']['thread']) > 0) {
            foreach($json['new']['thread'] as $thread) {
                $result[] = new CategorySummaryItem_Newest_Threads($thread);
            }         
        } 
        return $result;
    }
}

final class CategorySummaryItem_Popular_Threads extends CategorySummaryItem_ApiGetNewTop_Threads {
}

final class Category_Popular_Threads extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['top']) && isset($json['top']['thread']) && count($json['top']['thread']) > 0) {          
            foreach($json['top']['thread'] as $thread) {
                $result[] = new CategorySummaryItem_Popular_Threads($thread);
            }          
        } 
        return $result;
    }
}

abstract class CategorySummaryItem_ApiGetNewTop_Blogs extends CategorySummaryItem_ApiGetNewTop {
    public function parseJson($json) { 
        parent::parseJson($json);
        $this->id = $json['blogid'];  
        $this->updated_timestamp = $json['lastposttime'];        
    }
}

final class CategorySummaryItem_Newest_Blogs extends CategorySummaryItem_ApiGetNewTop_Blogs {
}
final class Category_Newest_Blogs extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['new']) && isset($json['new']['blog']) && count($json['new']['blog']) > 0) {        
            foreach($json['new']['blog'] as $blog) {
                $result[] = new CategorySummaryItem_Newest_Blogs($blog);
            }
            
        } 
        return $result;
    }
}

final class CategorySummaryItem_Popular_Blogs extends CategorySummaryItem_ApiGetNewTop_Blogs {
}

final class Category_Popular_Blogs extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['top']) && isset($json['top']['blog']) && count($json['top']['blog']) > 0) {           
            foreach($json['top']['blog'] as $blog) {
                $result[] = new CategorySummaryItem_Popular_Blogs($blog);
            }           
        } 
        return $result;
    }
}

abstract class CategorySummaryItem_ApiGetNewTop_Articles extends CategorySummaryItem_ApiGetNewTop {
    public function parseJson($json) { 
        parent::parseJson($json);
        $this->id = $json['nodeid'];
        $this->updated_timestamp = $json['publishtime'];        
    }
}

final class CategorySummaryItem_Newest_Articles extends CategorySummaryItem_ApiGetNewTop_Articles {
}

final class Category_Newest_Articles extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['new']) && isset($json['new']['article']) && count($json['new']['article']) > 0) {           
            foreach($json['new']['article'] as $article) {
                $result[] = new CategorySummaryItem_Newest_Articles($article);
            }           
        } 
        return $result;
    }
}

final class CategorySummaryItem_Popular_Articles extends CategorySummaryItem_ApiGetNewTop_Articles {
}

final class Category_Popular_Articles extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['top']) && isset($json['top']['article']) && count($json['top']['article']) > 0) {        
            foreach($json['top']['article'] as $article) {
                $result[] = new CategorySummaryItem_Popular_Articles($article);
            }          
        } 
        return $result;
    }
}

abstract class CategorySummaryItem_ApiGetNewTop_Photos extends CategorySummaryItem_ApiGetNewTop {
    protected $url_full;
    public final function getFullUrl() {
        return $this->url_full;
    }
    protected $contenturl;
    public final function getContentUrl() {
        return $this->contenturl;
    }
    
    public function getStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        return '<li class="category_data_item '.$name.' '.get_class().'" data-category="'.$name.'">
                    <a href="'.$this->getDetailViewLink($name).'"><img class="category_photo" class="photorestrict" src="'.vB_SITE_ROOT.'attachment.php?s='.VBApiClient::instance()->getUsersession().'&api=1&attachmentid='.$this->getId().'" /></a>
                </li>';
    }
    public function getSmallStreamHtml() {
        return $this->getStreamHtml();
    }
    
    public function getSingleSidebarHtml() {
        return $this->getStreamHtml();
    }
    
    public function parseJson($json) { 
        parent::parseJson($json);
        $this->id = $json['id'];
        $this->updated_timestamp = $json['dateline'];
        $this->url_full = $json['url_full'];
        $this->contenturl = $json['contenturl'];
    }
    
    protected function getDetailViewLink($category_name) {
        if(defined('READ_ONLY') && READ_ONLY) {
            return vB_SITE_ROOT . $this->getContentUrl();
        } else {
            return parent::getDetailViewLink($category_name);
        }
    }
}

final class CategorySummaryItem_Newest_Photos extends CategorySummaryItem_ApiGetNewTop_Photos {
}

final class Category_Newest_Photos extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['new']) && isset($json['new']['picture']) && count($json['new']['picture']) > 0) {           
            foreach($json['new']['picture'] as $picture) {
                $result[] = new CategorySummaryItem_Newest_Photos($picture);
            }           
        } 
        return $result;
    }
}

final class CategorySummaryItem_Popular_Photos extends CategorySummaryItem_ApiGetNewTop_Photos {
}

final class Category_Popular_Photos extends Category_ApiGetNewTop {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('api_getnewtop', array(), array('timestamp' => $this->getUpdatedCutoff(), 'max_items_per_category' => $this->getMaxItemsPerCategory())), true);
        $result = array();
        if(isset($json['top']) && isset($json['top']['picture']) && count($json['top']['picture']) > 0) {        
            foreach($json['top']['picture'] as $picture) {
                $result[] = new CategorySummaryItem_Popular_Photos($picture);
            }          
        } 
        return $result;
    }
}

abstract class CategorySummaryItem_Threadbits extends CategorySummaryItem {
    protected $replycount;
    protected $forumid;
    protected $forumtitle;
    protected $moved;
    protected $prefix;
    public final function getForumTitle() {
        return $this->forumtitle;
    }
    public final function getForumId() {
        return $this->forumid;
    }
    public final function getReplyCount() {
        return $this->replycount;
    }
    public final function getMovedPrefix(){
        return $this->moved;
    }
    public final function getPrefix(){
        return $this->prefix;
    }
    public function getStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        $title = $this->getTitle();
        $thread_prefix = isset($this->prefix) ? $this->prefix : "";
        if($this->getMovedPrefix()) {
            $json_forumid = json_decode(VBApiClient::instance()->call_cached('showthread', array('threadid' => $this->getId(), 'pagenumber' => 1, 'apitextformat' => 1 ), array()), true);
            $forumId = @$json['response']['thread']['forumid'];
            if(isset($forumId)) {
                $nameArray = explode("_", $name);
                $name = $nameArray[0] . "_" . $nameArray[1] . "_" . $forumId;
            }
        }
        return '<li class="category_data_item '.$name.' '.get_class($this).'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'&goto=newpost"><span class="thread_prefix">'. $this->getMovedPrefix() . $thread_prefix . '</span>' . $title.'</a>
                    </span>
                    
                    <div class="bottom_line">
                        <div class="teal_box">
                            <div class="teal_box_title">in</div><div class="teal_box_arrow"><div></div></div>
                            <div class="teal_box_text"><a href="category_stream.php?category=Forum_Threads_'.$this->getForumId().'">'.$this->getForumTitle().'</a></div>
                        </div>
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $this->getUsername() .'</a></div>
                    </div>
                    <span class="count view">
                        <span class="number">'. $this->getViewCount() .'</span>
                        <span class="text">views</span>
                    </span>
                    <span class="count reply">
                        <span class="number">'. $this->getReplyCount() .'</span>
                        <span class="text">replies</span>
                    </span>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function getSmallStreamHtml() {
        $name = Category::registry($this->getCategory())->getCategoryName();
        $threadTitle = $this->getTitle();
        $forumTitle = $this->getForumTitle();
        $thread_prefix = isset($this->prefix) ? $this->prefix : "";
        if($this->getMovedPrefix()) {
            $json_forumid = json_decode(VBApiClient::instance()->call_cached('showthread', array('threadid' => $ $this->getId(), 'pagenumber' => 1, 'apitextformat' => 1 ), array()), true);
            $forumId = @$json['response']['thread']['forumid'];
            if(isset($forumId)) {
                $nameArray = explode("_", $name);
                $name = $nameArray[0] . "_" . $nameArray[1] . "_" . $forumId;
            }
        }
        return '<li class="category_data_item small '.$name.' '.get_class().'" data-category="'.$name.'">
                    <span class="title">
                        <a href="'.$this->getDetailViewLink($name).'&goto=newpost"><span class="thread_prefix">'. $this->getMovedPrefix() . $thread_prefix . '</span>' .$threadTitle.'</a>
                    </span>                
                    <div class="bottom_line">
                        <div class="teal_box">
                            <div class="teal_box_title">in</div><div class="teal_box_arrow"><div></div></div>
                            <div class="teal_box_text"><a href="category_stream.php?category=Forum_Threads_'.$this->getForumId().'">'.$forumTitle.'</a></div>
                        </div>
                        <div class="username">'. timedisplay($this->getUpdatedTimestamp()) .' by <a href="'.$this->getProfileUrl().'">'. $this->getUsername() .'</a></div>
                    </div>
                    <span class="avatar"><a href="'.$this->getProfileUrl().'"><img src="'. $this->getAvatarUrl() .'" /></a></span>
                </li>
                ';
    }
    
    public function parseJson($json) {
        $this->id = $json['thread']['threadid'];
        $this->username = $json['thread']['lastposter'];
        $this->userid = @$json['thread']['lastposterid'];
        $this->title = $json['thread']['threadtitle'];
        $this->viewcount = $json['thread']['views'];
        $this->replycount = $json['thread']['replycount'];
        $this->updated_timestamp = $json['thread']['lastposttime'];
        $this->forumid = $json['thread']['forumid'];
        $this->forumtitle = $json['thread']['forumtitle'];
        if(!empty($json['avatar'])) {
            $this->avatarurl = $json['avatar']['0'];
        }
        if(!empty($json['thread']['movedprefix'])) {
            $this->moved = $json['thread']['movedprefix'];
        }
        else { 
            $this->moved = false;
        }
        if(!empty($json['thread']['prefix_rich'])) {
            $this->prefix = $json['thread']['prefix_rich'];
        }
        $forum = ForumInfo::get($this->forumid);
        $forum->setName($this->forumtitle);
        ForumInfo::set($forum);
        $thread = ThreadInfo::get($this->id);
        $thread->setName($this->title);
        ThreadInfo::set($thread);
    }
}

final class CategorySummaryItem_Subscribed_Threads extends CategorySummaryItem_Threadbits {
    public function parseJson($json) { 
        parent::parseJson($json);
        $forum = ForumInfo::get($this->forumid);
        $forum->setHasSubscribedThread(true);
        ForumInfo::set($forum);
        $thread = ThreadInfo::get($this->id);
        $thread->setSubscribed(true);
        $thread->setName($this->title);
        ThreadInfo::set($thread);
    } 
}

final class Category_Subscribed_Threads extends Category {
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('subscription_viewsubscription', array(), array('folderid' => 'all', 'perpage' => '30', 'page' => '1', 'sortfield' => 'lastpost', 'sortorder' => 'desc')), true);
        $result = array();
        if(isset($json['response']['HTML']['threadbits']) && count($json['response']['HTML']['threadbits']) > 0) {        
                if(empty($json['response']['HTML']['threadbits']['thread'])) {
                    foreach($json['response']['HTML']['threadbits'] as $thread) {
                        $result[] = new CategorySummaryItem_Subscribed_Threads($thread);
                    } 
                }  else {
                    $result[] = new CategorySummaryItem_Subscribed_Threads($json['response']['HTML']['threadbits']);
                }                  
            } 
        return $result;
    }
}

final class CategorySummaryItem_Recommended_Threads  extends CategorySummaryItem_Threadbits {
    public function parseJson($json) { 
        parent::parseJson($json);    
    }
}

final class Category_Recommended_Threads extends Category {
    protected function refreshData() {
        $result = array();
        $interests = $_SESSION['my_interests'];
        if(empty($interests['data'])) { return $result; }
        $rand_interest_name = $interests['data'][array_rand($interests['data'])]['name'];       
        $json = json_decode(VBApiClient::instance()->call_cached('search_process', array(), array( 'query' => $rand_interest_name, 
                                                                                                    'titleonly' => '0', 
                                                                                                    'searchdate' => ''. ($this->getRelevantPeriod() / (24 * 60 * 60)),
                                                                                                    'beforeafter' => 'after',
                                                                                                    'contenttypeid' => '1', 
                                                                                                    'search_type' => '1', 
                                                                                                    'sortby'  => 'relevance', 
                                                                                                    'order' => 'descending')), true);
        
        if($json['response']['errormessage'] == 'search') {
            $json = json_decode(VBApiClient::instance()->call_cached('search_showresults', array(), array('searchid' => $json['show']['searchid'])), true);
            if(isset($json['response']['searchbits']) && count($json['response']['searchbits']) > 0) {        
                if(empty($json['response']['searchbits']['thread'])) {
                    foreach($json['response']['searchbits'] as $thread) {
                        $result[] = new CategorySummaryItem_Recommended_Threads($thread);
                    } 
                }  else {
                    $result[] = new CategorySummaryItem_Recommended_Threads($json['response']['searchbits']);
                }                  
            } 
        }
        return $result;     
    }
}

abstract class CategorySummaryItem_Blogbits  extends CategorySummaryItem {   
    public function parseJson($json) { 
        $this->id = $json['blog']['blogid'];
        $this->username = $json['blog']['username'];
        $this->userid = $json['blog']['userid'];
        $this->title = $json['blog']['title'];
        $this->updated_timestamp = $json['blog']['time'];
        if(!empty($json['avatar'])) {
            $this->avatarurl = $json['avatar']['0'];
        }
    }
}
abstract class CategorySummaryItem_Articlebits  extends CategorySummaryItem {
    public function parseJson($json) { 
        $this->id = $json['article']['nodeid'];
        $this->username = $json['article']['username'];
        $this->userid = $json['article']['userid'];
        $this->title = $json['article']['title'];
        $this->updated_timestamp = $json['article']['publishtime'];
        if(!empty($json['avatar'])) {
            $this->avatarurl = $json['avatar']['0'];
        }
    }
}

final class CategorySummaryItem_Friends_Threads  extends CategorySummaryItem_Threadbits {
    public function parseJson($json) { 
        parent::parseJson($json);
    }
}

final class CategorySummaryItem_Friends_Blogs  extends CategorySummaryItem_Blogbits {
    public function parseJson($json) { 
        parent::parseJson($json);   
    }
}

final class CategorySummaryItem_Friends_Articles  extends CategorySummaryItem_Articlebits {
    public function parseJson($json) { 
        parent::parseJson($json);   
    }
}

class Category_Friends_All extends Category {
    public function getDisplayName() {
        return "Friends' Activity";
    }
    protected function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('get_vbfromfacebook', array(), array( 'facebookidList' => implode(',',$_SESSION['my_friends_ids']))), true);
        if(isset($json['response']['users']) && count($json['response']['users']) > 0) {
            $friendlist = $this->getVBUserIds($json['response']['users']);
            if(count($friendlist) > 0) {
                $json2 = json_decode(VBApiClient::instance()->call_cached('search_findusers', array(), array( 'userids' => implode(',',$friendlist), 'contenttypeids' => '1,15,18')), true);
                if(isset($json2) && $json2['response']['errormessage'] == 'search') {
                    $json3 = json_decode(VBApiClient::instance()->call_cached('search_showresults', array(), array('searchid' => $json2['show']['searchid'])), true);
                    return $this->parseSearchResults($json3);
                }
                return array();
            }
        }
        return array();              
    }
    
    private function getVBUserIds($vb_response) {
        $result = array();
        foreach($vb_response as $friend) {
            if(isset($friend['userid']))
            {
                $userid = $friend['userid'];
                $username = $friend['username'];
                $user_info = UserInfo::get($username);
                $user_info->setFBUserId($friend['fbuserid']);
                $user_info->setVBUserId($userid);
                $user_info->setIsFBFriend(true);
                UserInfo::set($user_info);
                
                $result[] = $userid;
            }
        }
        return $result;
    }
    
    private function parseSearchResults($json2) {
        
        $results = array();
        if(isset($json2['response']['searchbits']) && count($json2['response']['searchbits']) > 0) {        
            foreach($json2['response']['searchbits'] as $thread) {                       
                if(!empty($thread['thread']) && !$this->shouldFilter('thread'))
                    $results[] = new CategorySummaryItem_Friends_Threads($thread);
                if(!empty($thread['blog']) && !$this->shouldFilter('blog'))
                    $results[] = new CategorySummaryItem_Friends_Blogs($thread);
                if(!empty($thread['article']) && !$this->shouldFilter('article'))
                    $results[] = new CategorySummaryItem_Friends_Articles($thread);
                    
            }                       
        }
        return $results;
    }
    
    protected function shouldFilter($type) {
        return false;
    }
}

final class Category_Friends_Blogs extends Category_Friends_All {
    public function getDisplayName() {
        return "Friends' Blogs";
    }
    protected function shouldFilter($type) {
        if($type != 'blog') return true;
        return false;
    }
}

final class Category_Friends_Articles extends Category_Friends_All {
    public function getDisplayName() {
        return "Friends' Articles";
    }
    protected function shouldFilter($type) {
        if($type != 'article') return true;
        return false;
    }
}

final class Category_Friends_Threads extends Category_Friends_All {
    public function getDisplayName() {
        return "Friends' Threads";
    }
    protected function shouldFilter($type) {
        if($type != 'thread') return true;
        return false;
    }
}

// Stub classes to help with rendering

final class Category_All_Threads extends Category {
    protected function refreshData() {
        return array();
    }
}
final class Category_All_Photos extends Category {
    protected function refreshData() {
        return array();
    }
}
final class Category_All_Articles extends Category {
    protected function refreshData() {
        return array();
    }
}
final class Category_All_Blogs extends Category {
    protected function refreshData() {
        return array();
    }
}
final class Category_Popular_All extends Category {
    public function getDisplayName() {
        return "Popular Activity";
    }
    protected function refreshData() {
        return array();
    }
}
final class Category_Newest_All extends Category {
    public function getDisplayName() {
        return "Newest Activity";
    }
    protected function refreshData() {
        return array();
    }
}

final class Category_Recommended_All extends Category {
    public function getDisplayName() {
        return "Recommended Activity";
    }
    protected function refreshData() {
        return array();
    }
}
final class Category_Subscribed_All extends Category {
    public function getDisplayName() {
        return "Subscribed Activity";
    }
    protected function refreshData() {
        return array();
    }
}
final class Category_All_All extends Category {
    public function getDisplayName() {
        return "All Activity";
    }
    protected function refreshData() {
        return array();
    }
}

class CategorySummaryItem_Forum_Threads extends CategorySummaryItem_Threadbits {
    protected $forumid;
    public function __construct($json, $forumid) {
        $this->forumid = $forumid;
        parent::__construct($json);
    }
    public function getCategory() {
        return 'Forum_Threads_'. $this->forumid;
    }
}

final class CategorySummaryItem_Forum_Threads_Sticky extends CategorySummaryItem_Forum_Threads {
    private $addStickyScript;
    public function __construct($json, $forumid) {
        parent::__construct($json, $forumid);
        $this->addStickyScript = '
            <script>
                $(function() {
                    if($(".'.get_class($this).'").find(".title .contenttype").length < 1) {
                        $(".'.get_class($this).'").children(".title").prepend("<span class=\"contenttype\">sticky: </span>");
                    }
                });
            </script>
        ';
    }
    public function getSmallStreamHtml() {
        return parent::getSmallStreamHtml() . $this->addStickyScript;
    }
    public function getStreamHtml() {
        return parent::getStreamHtml() . $this->addStickyScript;
    }
}

abstract class Category_Paged extends Category {
    protected $page_nav;
    protected $page_num;
    
    protected function setPageNav($json) {
        $this->page_nav = PageNav::get($json);
        $_SESSION[$this->getCategoryName() . 'page_nav'][$this->page_num] = $this->page_nav;
    }
    
    protected function getPageNav() {
        if(!isset($this->page_nav)) {
            $this->page_nav = $_SESSION[$this->getCategoryName() . 'page_nav'][$this->page_num];
        }
        return $this->page_nav;
    }
    
    public function getSingleItemContainerHtml($inner) {
        return parent::getDashboardContainerHtml($inner);
    }
    
    public function getDashboardContainerHtml($inner) {
        $page_nav = $this->getPageNav();
        if(isset($page_nav)) {
            $html = '<div class="menu"><div class="paged_pages">'.$this->getPageNav()->getHtml() .'</div><div class="clear"></div></div>';
            return parent::getDashboardContainerHtml($html . '<ul>' . $inner . '</ul>' . $html);
        }    
        return parent::getDashboardContainerHtml($inner);
    }
    
    //
    //    Invalidate each page
    //
    public function invalidate() {
        $this->last_data_time = 0;
        $this->data = null;
        foreach($_SESSION[$this->sessionNamespace()] as $pagenum => &$page_array) {            
            $page_array['last_data_time'] = $this->last_data_time;
            $page_array['data'] = $this->data;
        }
    }
    
    //
    //  Cache each page
    //   
    protected function getCachedData() {
        $this->page_num = (!empty($_GET['page']) ? $_GET['page'] : 1) . '';
        if(isset($_SESSION[$this->sessionNamespace()][$this->page_num])) {
            $cat_session = $_SESSION[$this->sessionNamespace()][$this->page_num];
            $this->data = $cat_session['data'];
            $this->last_data_time = $cat_session['last_data_time'];
        }
    }
    protected function setData($data) {
        $this->last_data_time = self::now();
        $this->data = $data;
        $_SESSION[$this->sessionNamespace()][$this->page_num]['last_data_time'] = $this->last_data_time;
        $_SESSION[$this->sessionNamespace()][$this->page_num]['data'] = $this->data;
    }
}

final class Category_Forum_Threads extends Category_Paged {
    private $forumid;
    private $forumname;
    private $sticky_threads;
    private $is_sub;
    private $can_start_thread;
    
    protected function __construct($forumid, $weight = 0.5, $cache_data_for = 25, $relevant_period = 2592000, $color = '000000') {
        parent::__construct($forumid, $weight, $cache_data_for, $relevant_period, $color);
        $this->forumid = $forumid;
        $this->forumname = @$_SESSION[get_class($this) . $this->forumid . 'forumname'];
        $this->sticky_threads = @$_SESSION[get_class($this) . $this->forumid . 'sticky_threads'];
        $this->is_sub = @$_SESSION[get_class($this) . $this->forumid . 'is_sub'];
        $this->can_start_thread = @$_SESSION[get_class($this) . $this->forumid . 'can_start_thread'];
    }
    public function getDisplayName() {
        return $this->forumname;
    }  
    public function getContenttype() {
        return 'Threads';
    }
    
    public function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('forumdisplay', array('forumid' => $this->forumid, 'pagenumber' => $this->page_num), array()), true);
        parent::setPageNav($json['response']);

        $_SESSION[get_class($this) . $this->forumid . 'forumname'] = $this->forumname = $json['response']['foruminfo']['title_clean'];
        $_SESSION[get_class($this) . $this->forumid . 'is_sub'] = $this->is_sub = ($json['show']['subscribed_to_forum'] == 1);
        $_SESSION[get_class($this) . $this->forumid . 'can_start_thread'] = $this->can_start_thread = ($json['response']['forumrules']['can']['postnew'] == 1);
        
        $forum = ForumInfo::get($this->forumid);
        $forum->setName($this->forumname);
        $forum->setSubscribed($this->is_sub);
        ForumInfo::set($forum);
        
        $result = array();
        $this->sticky_threads = array();
        
        if(!empty($json['response']['threadbits_sticky'])) {
            if(array_key_exists('0', $json['response']['threadbits_sticky']))
            foreach($json['response']['threadbits_sticky'] as $thread) {
                if(isset($thread['thread'])) {
                    $this->sticky_threads[] = new CategorySummaryItem_Forum_Threads_Sticky($thread, $this->forumid);
                }
            }
            else $this->sticky_threads[] = new CategorySummaryItem_Forum_Threads_Sticky($json['response']['threadbits_sticky'], $this->forumid);
        }
        if(!empty($json['response']['threadbits'])) {
            if(array_key_exists('0', $json['response']['threadbits']))
            foreach($json['response']['threadbits'] as $thread) {
                if(isset($thread['thread'])) {
                    $result[] = new CategorySummaryItem_Forum_Threads($thread, $this->forumid);
                }
            }
            else $result[] = new CategorySummaryItem_Forum_Threads($json['response']['threadbits'], $this->forumid);
        }
        $_SESSION[get_class($this) . $this->forumid . 'sticky_threads'] = $this->sticky_threads;
        return $result;
    }
    
    public function getSingleItemContainerHtml($inner) {
        return parent::getSingleItemContainerHtml($inner);
    }
    
    public function getDashboardContainerHtml($inner) {      
        $page_nav = $this->getPageNav();  
          $bottom_html = '';
        $top_html =        '
            <div class="menu">
                <div class="subscribe button"><a onclick="changeForumSub('.$this->forumid.');">'.(!$this->is_sub ? 'Subscribe' : 'Unsubscribe').'</a></div>';
        if($this->can_start_thread) {
            $top_html .= '<div class="newthread button"><a '.((defined('READ_ONLY') && READ_ONLY) ? 'href="'.vB_SITE_ROOT.'newthread.php?do=newthread&f='.$this->forumid.'"' : 'onclick="Editor.show();"').'>Start new thread</a></div> ';
        }
                
        if(isset($page_nav)) {
            $top_html .= '<div class="paged_pages">'.$this->getPageNav()->getHtml() .'</div>';
            $bottom_html = '<div class="menu"><div class="paged_pages">'.$this->getPageNav()->getHtml() .'</div><div class="clear"></div></div>';
        }
        if($this->can_start_thread && (!defined('READ_ONLY') || !READ_ONLY)) {
            $top_html .= '<div class="clear"></div>' . Editor::factory($this->getCategoryName(), $this->forumid, 'Forums')->getHtml() .'</div>';
        }
        $top_html .= '<div class="clear"></div>';
        $start = '';
        foreach($this->sticky_threads as $thread) {
            $start = $start . $thread->getDashboardHtml();        
        }                 
        return Category::getDashboardContainerHtml($top_html . '<ul>' . $start . $inner . '</ul>' . $bottom_html);
    }
}
final class CategorySummaryItem_Search_Threads extends CategorySummaryItem_Threadbits {
    private $searchid;
    public function __construct($json, $searchid) {
        $this->searchid = $searchid;
        parent::__construct($json);
    }
    public function getCategory() {
        return 'Search_Threads_' . $this->searchid;
    }
}
final class CategorySummaryItem_Search_Blogs extends CategorySummaryItem_Blogbits {
    private $searchid;
    public function __construct($json, $searchid) {
        $this->searchid = $searchid;
        parent::__construct($json);
    }
    public function getCategory() {
        return 'Search_Blogs_' . $this->searchid;
    }
}
final class CategorySummaryItem_Search_Articles extends CategorySummaryItem_Articlebits {
    private $searchid;
    public function __construct($json, $searchid) {
        $this->searchid = $searchid;
        parent::__construct($json);
    }
    public function getCategory() {
        return 'Search_Articles_' . $this->searchid;
    }
}


class Category_Search_All extends Category_Paged {
    private $searchid;
    protected function __construct($searchid, $weight = 0.5, $cache_data_for = 25, $relevant_period = 2592000, $color = '000000') {
        parent::__construct($searchid, $weight, $cache_data_for, $relevant_period, $color);
        $this->searchid = $searchid;
    }
    public function refreshData() {
        $json = json_decode(VBApiClient::instance()->call_cached('search_showresults', array(), array('searchid' => $this->searchid, 'pagenumber' => $this->page_num)), true);
        if(@$json['response']['errormessage'][0] == 'searchnoresults' && !empty($this->data)) {
            return $this->data;
        }
        
        $criteria = @$json['response']['criteriaDisplay'];
        if($criteria) {
            $_SESSION[$this->getCategoryName() . 'display_name'] = $this->display_name = $this->parseCriteria($criteria);
        } else {
            $_SESSION[$this->getCategoryName() . 'display_name'] = $this->display_name = $this->parseCriteria2(json_encode($json));
        }      
        parent::setPageNav($json);
        return $this->parseSearchResults($json);
    }

    public function getDisplayName() {
        if(!isset($this->display_name)) {
            $this->display_name = $_SESSION[$this->getCategoryName() . 'display_name'];            
        }
        return $this->display_name;
    }
    
    protected function getIcon() {
        return '_img/ico_all.png';
    }
            
    private function parseSearchResults($json) {
        $results = array();
        if(isset($json['response']['searchbits']) && count($json['response']['searchbits']) > 0) { 
            if(    empty($json['response']['searchbits']['thread']) && 
                empty($json['response']['searchbits']['blog']) &&
                empty($json['response']['searchbits']['article'])) {
                    foreach($json['response']['searchbits'] as $searchitem) {                       
                        if(!empty($searchitem['thread']))
                            $results[] = new CategorySummaryItem_Search_Threads($searchitem, $this->searchid);
                        if(!empty($searchitem['blog']))
                            $results[] = new CategorySummaryItem_Search_Blogs($searchitem, $this->searchid);
                        if(!empty($searchitem['article']))
                            $results[] = new CategorySummaryItem_Search_Articles($searchitem, $this->searchid);
                            
                    } 
            } else {
                $searchitem = $json['response']['searchbits'];
                if(!empty($searchitem['thread']))
                    $results[] = new CategorySummaryItem_Search_Threads($searchitem, $this->searchid);
                if(!empty($searchitem['blog']))
                    $results[] = new CategorySummaryItem_Search_Blogs($searchitem, $this->searchid);
                if(!empty($searchitem['article']))
                    $results[] = new CategorySummaryItem_Search_Articles($searchitem, $this->searchid);
            }
        }
        return $results;
    }
    
    protected function parseCriteria($criteria) {
        $matches = array();
        preg_match('/<u>(.*[a-z, A-Z]+.*)<\/u>/', $criteria, $matches);
        if(empty($matches[1])) {
            return 'Search';
        }
        return 'You searched for: ' . $matches[1];
    }
    protected function parseCriteria2($search_string) {
        $matches = array();
        preg_match('/"&highlight=([a-z, A-Z]+)"/', $search_string, $matches);
        if(empty($matches[1])) {
            return 'Search';
        }
        return 'You searched for: ' . $matches[1];
    }
}

final class Category_Search_Threads extends Category_Search_All {
}
final class Category_Search_Blogs extends Category_Search_All {
}
final class Category_Search_Articles extends Category_Search_All {
}

//
//    User_Profile
//
//        User_Profile is a very specialized category for the purpose
//        of displaying a list of the current user's subscriptions
//        as well as a list of their facebook friends who use
//        this VB install.
//
//        This is somewhat frankenstein code, refreshData doesn't cleanly
//        map to one good data set, so we use the data set for
//        subscribed threads, and get the other data out of
//        the UserInfo cache and ForumInfo cache.
//        
final class Category_User_Profile extends Category {    
    public function refreshData() {
    
        //
        // Ensure our friends' data is up to date
        //
        $fcat = Category::registry('Friends_All');
        $fcat->getData();
        
        //
        //    Get forum subscription data
        //
        ForumInfo::refreshAll();
        
        return Category::registry('Subscribed_Threads')->getData();
    }
    
    protected function getIcon() {
        return "_img/ico_friends.png";
    }
    
    public function getDisplayName() {
        return "Your profile";
    }
    public function showAsFilter() {
        return false;
    }
    
    public function getSingleItemContainerHtml($inner) {
        return parent::getSingleItemContainerHtml($inner);
    }
    
    public function getDashboardContainerHtml($inner) {      
          $bottom_html = '';
        $top_html = '';
        $start = '';
        
        $toshow_friends = array();
        foreach(UserInfo::all() as $user) {
            if($user->isFBFriend()) {
                $toshow_friends[] = $user;                
            }
        }
        
        if(!empty($toshow_friends)) {
            $start .= '    <li class="profileheader"><div>Facebook friends using '.FACEBOOK_APP_NAME.'</div></li>
                        <li class="fbfriends"><div>';
            foreach($toshow_friends as $user) {
                $start .= '<a href="'.$user->getFBProfileUrl().'"><img src="'.$user->getFBAvatarUrl().'" /></a>';
            }
            $start .= '</div><div><a href="category_stream.php?category=Friends_All">See their activity</a></div></li>';
        }
        
        $toshow_forums = array();
        foreach(ForumInfo::all() as $forum) {
            if($forum->isSubscribed()) {
                $toshow_forums[] = $forum;                
            }
        }
        
        if(!empty($toshow_forums)) {
            $start .= '    <li class="profileheader"><div>Subscribed Forums</div></li>';
            foreach($toshow_forums as $forum) {
                $start .= $this->getForumSubscriptionHtml($forum->getName(), $forum->getId());
            }
        }
        $this->getData();
        if(!empty($this->data)) {
            $start .= '    <li class="profileheader"><div>Subscribed Threads</div></li>';
            foreach($this->data as $thread) {
                $start .= $this->getThreadSubscriptionHtml($thread->getTitle(), $thread->getId());
            }
        }
        
        if (isset($_SESSION['user_credentials'])) {
            $start .= '<li class="fbfriends"><div><a href="logout.php">Log out of your account</a></div></li>';
        }
        
        return Category::getDashboardContainerHtml($start);
    }
    
    private function getForumSubscriptionHtml($name, $id) {
        return '<li class="category_data_item profile forum'.$id.'">
                    <div class="title"><a href="category_stream.php?category=Forum_Threads_'.$id.'">'.$name.'</a></div>
                    <div class="menu">
                        <div class="subscribe button"><a onclick="profileUnsub(\'forum\', '.$id.');">Unsubscribe</a></div>
                    </div>
                </li>';
    }
    
    private function getThreadSubscriptionHtml($name, $id) {
        return '<li class="category_data_item profile thread'.$id.'">
                    <div class="title"><a href="category_item.php?category=Subscribed_Threads&id='.$id.'&goto=newpost">'.$name.'</a></div>
                    <div class="menu">
                        <div class="subscribe button"><a onclick="profileUnsub(\'thread\', '.$id.');">Unsubscribe</a></div>
                    </div>
                </li>';
    }
}

//
//    - Category, CategorySummaryItem Implementations -
//

//    
//    CategoryContentItem
//
//        This class implements the functionality of the single item view.
//        It is responsible for knowing how to acquire the data and view html
//        for a given content type, e.g. Threads.
//
//        This base class handles the caching of the data, as well as providing
//        a common interface for the front end code to consume.
//        
//        Typicaly each subclass will implement __construct($id, $page = default)
//        which will populate the $this->html string with the html representation of
//        the data it acquires from the API in the same function.
//        
abstract class CategoryContentItem {
    protected $html = "<div class=\"content_item\">This content item does not exist, or you do not have permission to view this content item.</div>";
    private static function session_namespace() {
        return get_class();
    }
    public static function get($category, $id, $page = 1, $can_be_this_old = 2) {
        $contenttype = Category::registry($category)->getContenttype();
        if(!isset($_SESSION[self::session_namespace()][$category][$id][$page]) || 
            (time() - $_SESSION[self::session_namespace()][$category][$id][$page]['time']) > $can_be_this_old) {
            $class_name = get_class() . '_' . $contenttype;
            $_SESSION[self::session_namespace()][$category][$id][$page]['data'] = new $class_name($category, $id, $page);
            $_SESSION[self::session_namespace()][$category][$id][$page]['time'] = time();
        }
        return $_SESSION[self::session_namespace()][$category][$id][$page]['data'];
    }
    
    public final function getHtml() {
        return $this->html;
    }
    
    public static function getRedirect($category, $id, $page = 1) {
        $contenttype = Category::registry($category)->getContenttype();
        switch($contenttype) {
            case 'Threads':
                return 'showthread.php?t='.$id.'&page='.$page;
            case 'Blogs':
                return 'entry.php/'.$id;
            case 'Articles':
                return 'content.php/'.$id;
            case 'Photos':
                return 'attachment.php?s='.VBApiClient::instance()->getUsersession().'&api=1&attachmentid='.$id;
            default:
                return '';
        }   
    }
}

//
//    + CategoryContentItem implementations +
//

final class CategoryContentItem_Threads extends CategoryContentItem {
    protected function __construct($category, $id, $page = 1) {
        $json = json_decode(VBApiClient::instance()->call_cached('showthread', array('threadid' => $id, 'pagenumber' => $page, 'apitextformat' => 5), array()), true);
        $errormessage = (array_key_exists('errormessage', $json['response']))? $json['response']['errormessage'][0] : '';
        if($errormessage)
           return;
        $page_nav = PageNav::get($json);
        $page_nav_html = isset($page_nav) ? $page_nav->getHtml() : PageNav::defaultHtml();

        $thread = ThreadInfo::get($id);
        $thread->setSubscribed(@$json['show']['subscribed'] == 1);
        $thread->setName($json['response']['thread']['title']);
        ThreadInfo::set($thread);
        $forumname = ForumInfo::get($json['response']['thread']['forumid'])->getName();
        $forumname_html = '';
        if(isset($forumname)) {
            $forumname_html = '
                <div class="teal_box">
                    <div class="teal_box_title">in</div><div class="teal_box_arrow"><div></div></div>
                    <div class="teal_box_text"><a href="category_stream.php?category=Forum_Threads_'.$json['response']['thread']['forumid'].'">'.$forumname.'</a></div>
                </div>';
        }

        $hide_reply = (($json['show']['closethread'] === 0) && !($json['show']['managethread'] === 1 || $json['show']['adminoptions'] === 1));
        $threadOptions = '<div class="reply button"><a onclick="Editor.show();">Reply</a></div>';
        if($hide_reply) $threadOptions = '<div class="reply button"><span>Closed</span></div>';

        $this->html = '';
        $this->html = $this->html .
            '<div class="content_item thread">
                <div class="top">
                <div class="title">'.$json['response']['thread']['title'].'</div>
                <div class="menu">
                <div class="subscribe button"><a onclick="changeThreadSub('.$id.');">'.((@$json['show']['subscribed'] != 1) ? 'Subscribe' : 'Unsubscribe').'</a></div>
                '. $threadOptions . '</div>
                <div class="pagination clear">'. $forumname_html . $page_nav_html .'</div>
                </div>

            ';
        $this->html = $this->html . '<div class="posts">';
        foreach($json['response']['postbits'] as $key => $data) {
            if($key === 'post') {
                $this->displayPost($data, $hide_reply);
            } else if(is_numeric($key)) {
                $this->displayPost($data['post'], $hide_reply);
            }
        }
        $this->html = $this->html . '</div>';
        if(!$hide_reply) {
            $this->html = $this->html . Editor::factory($category, $id)->getHtml();
        }
        $this->html = $this->html . '<div class="pagination clear">'. $page_nav_html .'</div>';
        $this->html = $this->html . '</div>
            <script>
                $(function() {
                    $(".post").last().css({border: "none"});
                    $(".postmessage").find("*").each(function() {
                        var $this = $(this);
                        if($this.outerWidth(true) > 440) {
                            $this.css({width: "440px"});
                        }
                    });
                    $(".postmessage").find("img").css({maxWidth: "440px"});
                });
            </script>
        ';
        if(@$_GET['goto'] == 'newpost') {
            $this->html = $this->html . '
                <script>
                    var int;
  		    $(window).load(function(){ 
                        if(window.location.hash !== "#post" + '. $json['response']['LASTPOSTID'] .'){
                            int = setInterval("doHash(\"#post\"+ '. $json['response']['LASTPOSTID'] .')",1000);
                        }
                        window.location.hash = "#post" + '. $json['response']['LASTPOSTID'] .';

		    });

                function doHash(hash){
                    if(window.location.hash == hash && (typeof VBFB_Stream == "undefined")){
                        clearInterval(int);
                        return false;
                    }
                    window.location.hash = hash;
		    window.location.href = hash;
                }
                </script>
            ';
        }
    }
    
    private function displayPost($post_json, $isThreadClosed) {
        $user_info = UserInfo::get($post_json['username']);
        $user_info->setVBUsertitle($post_json['usertitle']);
        $user_info->setVBAvatarUrl(empty($post_json['avatarurl']) ? null : $post_json['avatarurl']);
        UserInfo::set($user_info);
        
        $avatarurl = $user_info->getVBAvatarUrl();
        $profilelink = $user_info->getVBProfileUrl();
        if($user_info->hasFBData()) {
            $avatarurl = $user_info->getFBAvatarUrl();
            $profilelink = $user_info->getFBProfileUrl();
        }

        $position = strpos($post_json['message'], "href=\"vb:");
        if($position !== false){
            $post_json['message'] = $this->changeMobileUrls($post_json['message']);
        }


        $threadOptions = '<div class="reply"><a onclick="Editor.show();">Reply</a></div><div class="quote"><a onclick="Editor.show(\''.$post_json['postid'].'\');">Quote</a></div>';
        if($isThreadClosed == true) $threadOptions = '';
        $this->html = $this->html . '
            <a name="post'.$post_json['postid'].'"></a>
            <div class="post '.$post_json['postid'].'">
                
                <div class="header">
                    <div class="username"><a href="'.$profilelink.'">'.$post_json['username'].'</a></div>
                    <div class="usertitle">'.$post_json['usertitle'].'</div>
                    <div class="avatar"><a href="'.$profilelink.'"><img src="'.$avatarurl.'" /></a></div>
                    <div class="posted">'.timedisplay($post_json['posttime']).'</div>
                </div>
                <div class="postmessage">'.@$post_json['message'].'</div>
                <div class="postmessage bbcode" data-postid="' . $post_json['postid'] . '">' . base64_encode($post_json['message_bbcode']) .'</div>
                <div class="postoptions">
                   ' . $threadOptions .'</div>
                <div class="clear"></div>
            </div>';
    }

    private function changeMobileUrls($message)
    {
        $tokens = explode('"', $message);
        for($i = 0 ; $i<count($tokens) ; $i++){
            $strPost = strpos($tokens[$i], 'href="vb:');
            if($strPost !== false){
                $url = $tokens[$i];
                $tokens[$i] = 'href="' . VbLinkHelper::translate($url) . '"';
            }

        }
        return implode('"', $tokens);

    }

}


final class CategoryContentItem_Blogs extends CategoryContentItem {
    protected function __construct($category, $id) {
        $json = json_decode(VBApiClient::instance()->call_cached('blog_blog', array('blogid' => $id, 'apitextformat' => 2), array()), true);
        $errormessage = (array_key_exists('errormessage', $json['response']))? $json['response']['errormessage'][0] : '';
        if($errormessage)
           return;
        $user_info = UserInfo::get(@$json['response']['content']['blog']['postedby_username']);
        $user_info->setVBAvatarUrl(@$json['response']['content']['blog']['avatarurl']);
        $user_info->setVBUserid(@$json['response']['content']['bloginfo']['userid']);
        UserInfo::set($user_info);
        
        $this->html = '
            <div class="blog content_item">
            <div class="blogheader">
                <div class="blogtitle">'.@$json['response']['content']['blog']['title'].'</div>
                <div class="blogdateline">
                    <div class="blogauthor">
                        by <a href="'.$user_info->getProfileUrl().'">'.@$json['response']['content']['blog']['postedby_username'].'</a>
                        on '.@date('D j M \'y \a\t g:ia', @$json['response']['content']['blog']['time']).'
                    </div>
                </div>
            </div>
            <div class="blogcontent">
                '.@$json['response']['content']['blog']['message'].'
            </div>
            <div class="blogfooter">
                <a href="'.$user_info->getProfileUrl().'"><img src="'.$user_info->getAvatarUrl().'" /></a>
                <div class="userinfo">
                    <div class="name"><a href="'.$user_info->getProfileUrl().'">'.@$json['response']['content']['blog']['postedby_username'].'</a></div>
                    <div class="title">'.$user_info->getVBUsertitle().'</div>
                </div>
            </div>
            </div>
        ';
    }
}

final class CategoryContentItem_Articles extends CategoryContentItem {
    protected function __construct($category, $id, $page = '') {
        $json = json_decode(VBApiClient::instance()->call_cached('cms.content_'.$id.'_view' . (!empty($page) ? '_' . $page : ''), array('apitextformat' => 2), array()), true);
        $errormessage = (array_key_exists('errormessage', $json['response']))? $json['response']['errormessage'][0] : '';
        if($errormessage)
           return;
        $user_info = UserInfo::get(@$json['response']['layout']['content']['username']);
        $user_info->setVBUserid(@$json['response']['layout']['content']['userid']);
        UserInfo::set($user_info);
        
        $page_widget = '';
        $hide_first = false;
        if(!empty($json['response']['layout']['content']['pagelist'])) {
            $page_widget = '<div><div class="pages"><span>Pages: </span><select class="pageselect">';            
            foreach(@$json['response']['layout']['content']['pagelist'] as $key => $value) {
                if($value['selected'] == 1 && $key == 1) {
                    $hide_first = true;
                }
                $page_widget = $page_widget . '<option'. (($value['selected'] == 1) ? ' selected="selected"' : '') .' data-pagenum="'.$key.'">' . $value['title'] . '</option>'; 
                $last_selected = $value['selected'];
            }
            
            $page_widget = $page_widget . '</select>';
            
            if(!$hide_first) {
                $page_widget = $page_widget . '<div class="prev"><a href="' . PageNav::getLink($page - 1) .'"><img src="_img/page_prev.png" /></a></div>';
            }
            if($last_selected != 1) {
                $page_widget = $page_widget . '<div class="next"><a href="' . PageNav::getLink($page + 1) .'"><img src="_img/page_next.png" /></a></div>';
            }
            
            $page_widget = $page_widget . '</div><div class="clear"></div></div>';
        }
        
        $publish_line = '';
        if(@$json['response']['layout']['content']['showpublishdate'] == 1) {
            $publish_line = $publish_line .    '<div class="publishedon">';
            if(@$json['response']['layout']['content']['published'] == 1) {
                $publish_line = $publish_line . 'Published on '.@date('D j M \'y \a\t g:ia', @$json['response']['layout']['content']['setpublish']);
            } else if(!empty($json['response']['layout']['content']['setpublish'])) {
                $publish_line = $publish_line . 'Will be published on '.@date('D j M \'y \a\t g:ia', @$json['response']['layout']['content']['setpublish']);
            } else {
                $publish_line = $publish_line . 'Not published';
            }
            $publish_line = $publish_line .    '</div>';
        }
        
        $this->html = '
            <div class="article content_item">
            <div class="articleheader">
                <div class="articletitle">'.@$json['response']['layout']['content']['title'].'</div>
                <div class="articledateline">
                    <div class="articleauthor">
                        by <a href="'.$user_info->getProfileUrl().'">'.@$json['response']['layout']['content']['username'].'</a>                    
                    </div>
                    '. $publish_line. '
                </div>
                '. $page_widget. '
            </div>
            <div class="articlecontent">
                '.@$json['response']['layout']['content']['pagetext'].'
            </div>
            <div class="articlefooter">
                <a href="'.$user_info->getProfileUrl().'"><img src="'.$user_info->getAvatarUrl().'" /></a>
                <div class="userinfo">
                    <div class="name"><a href="'.$user_info->getProfileUrl().'">'.@$json['response']['layout']['content']['username'].'</a></div>
                    <div class="title">'.$user_info->getVBUsertitle().'</div>
                </div>
            </div>
            </div>
            <script>
                $(".pages .pageselect").change(function(e) {
                    var vars = [], hash;
                    var hashes = window.location.href.slice(window.location.href.indexOf("?") + 1).split("&");
                    for(var i = 0; i < hashes.length; i++)
                    {
                        hash = hashes[i].split("=");
                        vars.push(hash[0]);
                        vars[hash[0]] = hash[1];
                    }
                    window.location.href = "category_item.php?category=" + vars["category"] + "&id='.$id.'&page=" + $(this).children(":selected").attr("data-pagenum");
                });
            </script>
        ';
    }
}

final class CategoryContentItem_Photos extends CategoryContentItem {
    protected function __construct($category, $id) {
        $this->html = '<a href="'.vB_SITE_ROOT.'attachment.php?s='.VBApiClient::instance()->getUsersession().'&api=1&attachmentid='.$id.'"><img src="'.vB_SITE_ROOT.'attachment.php?s='.VBApiClient::instance()->getUsersession().'&api=1&attachmentid='.$id.'" class="contentitemrestrict" /></a>';
    }
}

//
//    - CategoryContentItem implementations -
//

?>
